import os
import configparser

from importlib.metadata import version


# Set up globals
__version__ = version('xwordlist')

FILE = {
    'name': 'xwordlist',
    'conf': 'xwordlist.conf',
}

CONFIG = os.path.join(os.path.expanduser('~/.config/xwordlist'), FILE['conf'])

DEFAULTS = {
    'convert': '',
    'strip': 'diacritic',
    'minimum': 3,
    'case': 'upper',
    'dedupe': 'nocase',
    'webextract': 'text',
    'alphabetize': 'normal',
}

OPTIONS = {
    'strip': ['diacritic', 'keepdiacritic'],
    'case': ['upper', 'lower', 'none'],
    'dedupe': ['nocase', 'bycase'],
    'alphabetize': ['normal', 'reverse'],
}

GLOBAL_SETTINGS = {
    'urllist_delay': 20,
    'file_add': 'xwl',
    'impact_color': 'ansired',
    'parse_char': ';',
}

IMPACT_COLOR = 'ansired'
COLOR_OPTIONS = ['ansiblack', 'ansired', 'ansigreen', 'ansiyellow', 'ansiblue', 'ansimagenta',
                 'ansicyan', 'ansigray', 'ansibrightblack', 'ansibrightred', 'ansibrightgreen',
                 'ansibrightyellow', 'ansibrightblue', 'ansibrightmagenta', 'ansibrightcyan', 'ansiwhite']


def init_config():
    configClass = configparser.ConfigParser()
    configClass['globals'] = GLOBAL_SETTINGS
    configClass['defaults'] = DEFAULTS

    # First see if the config file exists and if not, create it
    if not os.path.exists(CONFIG):
        os.makedirs(os.path.dirname(CONFIG), exist_ok=True)

    # Otherwise read the file
    else:
        configClass.read_file(open(CONFIG))
        # Check to make sure there are no items which aren't correct (and revert to default if so)
        for key, item in configClass['defaults'].items():
            if key in OPTIONS and item not in OPTIONS[key]:
                configClass['defaults'][key] = DEFAULTS[key]

    # See if conf file contains an impact color
    global IMPACT_COLOR
    IMPACT_COLOR = configClass.get('globals', 'impact_color')

    # Now write a new file
    configClass.write(open(CONFIG, 'w'))

    # If directory option exists, expand it before passing back
    if 'directory' in configClass['globals']:
        configClass['globals']['directory'] = os.path.expanduser(configClass['globals']['directory'])

    return configClass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xwordlist']

package_data = \
{'': ['*']}

install_requires = \
['anyascii>=0.3.1,<0.4.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'prompt-toolkit>=3.0.20,<4.0.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['xwl = xwordlist.xwordlist:main',
                     'xwordlist = xwordlist.xwordlist:main']}

setup_kwargs = {
    'name': 'xwordlist',
    'version': '23.0.1',
    'description': 'Software to help create, build and organize crossword puzzle word lists',
    'long_description': '# xwordlist\n\n`xwordlist` is a command line Python program designed to help you create, build and organize crossword puzzle word lists. As I started to think about constructing crossword puzzles with heavy themes — trying to make the entire puzzle themed, not just three or four long entries — I realized that I would need the ability to acquire and organize large amounts of text on very specific topics. After hacking around with a combination of search-and-replace text editors and Excel, I realized I needed to build someting more custom and thus `xwordlist` was born. \n\nBesides helping with basic text functions such as deduping, alphabetizing and changing case, this program is able to pull content out of structured web pages and parse large blocks of text, including lists of web pages with similarly structured content. Although I first started using the software to grab the lyrics of songs, I have added regex and better html parsing functionality to make it easier to get data from Wikipedia and less structured sites.\n\nFor more information, see the project’s main website hosted at [xwl.ist](https://xwl.ist). For an example of a themed 5x5 mini puzzle built with a word list assembled using this software, see [my personal website](https://quid.pro/xwords/tom-petty-mini/). For one of the original inspirations for this project, see Malaika Handa’s [Crosswift](https://www.7xwords.com/daily/01/01-13.html) puzzle using only Taylor Swift’s lyrics.\n\n## Installation\n\nIt helps to have some familiarity with Python and terminal programs to install `xwordlist` but it is not a requirement. If your environment is all set up and you know `pip` is both installed and up-to-date, skip down to the `pip` instructions below. Most Macs and Linux users will find that `pip` is installed but probably needs to be updated. Most Windows users will need to install `pip` first (see [Windows Special Instructions](#windows-special-instructions) below). In other words, unless you really know what you are doing, you probably need to continue to follow the next paragraph or the special Windows instructions.\n\nSearch for `terminal` and your operating system should show you the name and how to launch your default terminal program. The first thing you will need to do is make sure your Python is up-to-date (required) and that you have activated a virtual environment (recommended). See [Installing Python Packages](https://packaging.python.org/en/latest/tutorials/installing-packages/) for helpful instructions on how to do both as well as how to install and upgrade `pip`. Follow the instructions on that page down to the section labeled *Installing from PyPI*.\n\nFrom there, you can install `xwordlist` by typing\n\n```\npip install xwordlist\n```\nBe sure to type `xwordlist` as there is other software called `wordlist` (without the “x”) which is not what you want. To see if your installation was successful, type\n```\nxwordlist --version\n```\nIf properly installed, the software should respond with its name and the installed version.\n\n### Windows Special Instructions\n\nRegardless of whether Python is already installed on your Windows computer, I would highly recommend a fresh install of an up-to-date version from [Python.org’s Windows downloads page](https://www.python.org/downloads/windows/). In the *Stable Releases* column are a number of Windows installers that are compatible with versions of Windows going back to XP and compatible with 32-bit and 64-bit Intel machines or 64-bit ARM. Find the most recent version of Python that is compatbile with your version of Windows and the type of processor you have and download that.\n\nWhen you launch the installer, select “Customize Installation” and make sure the boxes next to “Pip” and “Py launcher” are selected. Hit the “Next” button and make sure the boxes next to both “Create shortcuts for installed applications” and “Add Python to environment variables” are selected. Finally, hit the “Install” button.\n\nFrom there, you should be able to follow the original instructions above by typing\n\n`pip install xwordlist`\n\n### Manual Installation\n\nTo install the software manually, copy the `xwordlist` code to your local working environment by either cloning this repository or downloading and unpacking the [zip archive](https://github.com/aanker/xwordlist/archive/refs/heads/main.zip) into a new directory. To install the dependencies required to make `xwordlist` work, use your terminal program to find the directory in which you have copied the files and type\n\n```\npython3 -m pip install -r requirements.txt\n```\nTo see if your installation was successful, type\n```\npython3 xwordlist.py --version\n```\nIf properly installed, the software should respond with its name and the installed version.\n\n## Usage\n\nIf you have installed the software using `pip`, you should be able to run the program by simply typing `xwordlist` or `xwl`. For manual installs, you will need to type `python3 xwordlist.py`. The rest of the documentation assumes you have installed via `pip` and uses the short form.\n\nFor quick help instructions on the command line, type\n```\nxwordlist --help\n```\nPlease see the [project’s main website](https://xwl.ist) for more information about using the software including a [basic example](https://xwl.ist/help/#basic-example), [recipes for common patterns](https://xwl.ist/resources/#recipes) and a [reference](https://xwl.ist/help/#list-of-available-options) to all options.\n\nFind a bug? Please let us know [here](https://github.com/aanker/xwordlist/issues).\n\n## License\n\nThis software is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).\n',
    'author': 'Andrew Anker',
    'author_email': 'aa@quid.pro',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://xwl.ist',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import re
import requests
import urllib.parse

from bs4 import BeautifulSoup
from anyascii import anyascii


class WordList:
    def __init__(self, myList=[]):
        self.myList = myList

    # List transformation options
    def minimum(self, numChars):
        if isinstance(numChars, int) or numChars.isdigit():
            self.myList = [line for line in self.myList if line and len(line[0]) >= int(numChars)]
        else:
            err_dict = {
                'error': 'Exiting... argument for minimum must be an integer, not {}',
                'arg': numChars,
            }
            raise XWLException(err_dict)

    def strip(self, stripWhat):
        newList = []
        if stripWhat in ['keepdiacritic', 'diacritic']:
            for word, other in self.myList:
                if stripWhat != 'keepdiacritic':
                    word = anyascii(word)
                newWord = ''.join([i for i in word if i.isalpha()])
                if len(newWord) > 0:
                    newList.append([newWord, other])
            self.myList = list(newList)
        else:
            err_dict = {
                'error': 'Exiting... unknown strip option {}',
                'arg': stripWhat,
            }
            raise XWLException(err_dict)

    def case(self, newCase):
        case_dict = {
            'upper': str.upper,
            'lower': str.lower,
        }
        if newCase in case_dict:
            self.myList = [[case_dict[newCase](line[0]), line[1]] for line in self.myList]
        elif newCase != 'none':
            err_dict = {
                'error': 'Exiting... unknown case option {}',
                'arg': newCase,
            }
            raise XWLException(err_dict)

    def dedupe(self, dedupeType):
        if dedupeType in ['bycase', 'nocase']:
            newList = []
            newSet = set()
            for word, other in self.myList:
                dupe_check = word.casefold() if dedupeType == 'nocase' else word
                if dupe_check not in newSet:
                    newSet.add(dupe_check)
                    newList.append([word, other])
            self.myList = list(newList)
        else:
            err_dict = {
                'error': 'Exiting... incorrect dedupe option {}',
                'arg': dedupeType,
            }
            raise XWLException(err_dict)

    def alphabetize(self, direction):
        if direction == 'normal':
            self.myList.sort(key=lambda x: x[0].casefold())
        elif direction == 'reverse':
            self.myList.sort(key=lambda x: x[0].casefold(), reverse=True)
        else:
            err_dict = {
                'error': 'Exiting... incorrect alphabetize option {}',
                'arg': direction,
            }
            raise XWLException(err_dict)

    # Content parsing options
    def regex(self, regexInput):
        try:
            newList = []
            for line, other in self.myList:
                reg_find = re.findall(regexInput, line)
                for reg_line in reg_find:
                    newList.append([reg_line, other])
            self.myList = list(newList)

        except re.error:
            err_dict = {
                'error': 'Regex pattern {} not valid, please check and try again',
                'arg': regexInput,
            }
            raise XWLException(err_dict)

        except Exception as e:
            err_dict = {
                'error': 'Error {}',
                'arg': e,
            }
            raise XWLException(err_dict)

    def convert(self, parseChars):
        # Add a space to parseChars since we always parse by that
        parseChars += ' '
        for char in parseChars:
            newList = []
            for line, other in self.myList:
                if line.find(char) != -1:
                    line_find = line.split(char)
                    for new_line in line_find:
                        newList.append([new_line, other])
                else:
                    newList.append([line, other])
            self.myList = list(newList)


class WebExtract:
    PARSEDICT = {}
    WEBEXTRACT = ''

    def __init__(self, parseDict={}, webExtract=''):
        self.returnWords = []
        self.parseDict = parseDict if parseDict != {} else self.PARSEDICT
        self.webExtract = webExtract if webExtract != '' else self.WEBEXTRACT

    def pull_data(self, getData):
        return self._get_web_page(getData)

    def _get_web_page(self, webURL):
        try:
            r = requests.get(webURL)
            if r.status_code == 200:
                inputSoup = BeautifulSoup(r.text, 'html.parser')
                if self.parseDict:
                    # See if we have a class, in which case, have to do more screening (1 to N classes)
                    if 'class' in self.parseDict:
                        classDict = {}
                        classDict['class'], whichNum = self.parseDict['class']
                        fullSoup = inputSoup.find_all(attrs=classDict)
                        for counter, whichSoup in enumerate(fullSoup, start=1):
                            if whichNum == counter or whichNum == 0:
                                self._extract_from_web(whichSoup, webURL)
                    else:
                        self._extract_from_web(inputSoup.find(attrs=self.parseDict), webURL)
                else:
                    self._extract_from_web(inputSoup, webURL)

                for line in self.scrapeWords:
                    self.returnWords.append([line, ''])

            elif r.status_code == 403:
                err_dict = {
                    'error': 'Unable to load webpage due to code 403: this usually means we are being blocked',
                }
                raise XWLException(err_dict)

            else:
                err_dict = {
                    'error': 'Unable to load webpage, status code = {}',
                    'arg': r.status_code,
                }
                raise XWLException(err_dict)

        except XWLException:
            raise

        except (requests.URLRequired, requests.RequestException):
            err_dict = {
                'error': 'No content retrieved, error URL: {}',
                'arg': self.webURL,
            }
            raise XWLException(err_dict)

        except AttributeError:
            err_dict = {
                'error': 'HTML attribute {} not found, check document and try again',
                'arg': self.parseDict,
            }
            raise XWLException(err_dict)

        except Exception as e:
            err_dict = {
                'error': 'Web error {}',
                'arg': e,
            }
            raise XWLException(err_dict)

    def _extract_from_web(self, soup, extractURL):
        # A few ways the default option can come in, try that first
        if self.webExtract == 'text':
            self.scrapeWords = soup.stripped_strings
        elif self.webExtract == 'links':
            self.scrapeWords = []
            for link in soup.find_all('a'):
                getURL = link.get('href')
                if getURL:
                    parsePieces = urllib.parse.urlsplit(getURL)
                    # Check to see if absolute or relative URL.  If relative, make it absolute
                    if parsePieces.scheme == '' and parsePieces.netloc == '':
                        parseExtract = urllib.parse.urlsplit(extractURL)
                        getURL = urllib.parse.urljoin(f'{parseExtract.scheme}://{parseExtract.netloc}', getURL)
                    self.scrapeWords.append(getURL)
        elif self.webExtract[:5] == 'html-':
            self.scrapeWords = []
            extractTags = self.webExtract[5:].split('_')
            for tag in extractTags:
                for link in soup.find_all(tag):
                    text = link.get_text()
                    self.scrapeWords.append(text)
        else:
            error = f'Exiting... incorrect option for webextract: {self.err_text(self.webExtract)}'
            raise XWLException(error)


class XWLException(Exception):
    pass



#!/usr/bin/env python3

import argparse
import os
import sys
import pathlib
import urllib.parse
import time

from .xwl import WordList, WebExtract, XWLException
from .xwlconfig import __version__, FILE, IMPACT_COLOR, init_config
from prompt_toolkit import prompt, print_formatted_text, HTML


def print_line(printText, argument={}, endText='\n'):
    print_formatted_text(HTML(printText.format(**argument, color=IMPACT_COLOR)), end=endText)
    return


def create_dict(localAttrs):
    returnDict = {}
    for attr in localAttrs:
        dict_parts = attr.split('=')
        # See if it is a class dictionary, in which case we make a tuple
        if dict_parts[0] == 'class':
            numParts = int(dict_parts[2]) if len(dict_parts) == 3 else 0
            returnDict[dict_parts[0]] = (dict_parts[1], numParts)
        else:
            returnDict[dict_parts[0]] = dict_parts[1]
    if len(returnDict) > 0:
        return returnDict
    else:
        print_line('Exiting... something is wrong with your --container setting')
        sys.exit()


def get_file_content(inputFile, doParse, parseChar=''):
    try:
        with open(inputFile, 'r') as inputOpen:
            returnFile = []
            for line in inputOpen:
                if doParse and parseChar in line:
                    split = line.split(parseChar)
                    returnFile.append([split[0], split[1].strip()])
                else:
                    returnFile.append([line.strip(), ''])
            return returnFile

    except UnicodeDecodeError:
        print_line('Exiting... only text files accepted')
        sys.exit()

    except FileNotFoundError:
        error_text = 'Exiting... input file <{color}>{inputFile}</{color}> does not exist'
        print_line(error_text, {'inputFile': inputFile})
        sys.exit()

    except Exception as e:
        print_line('Exiting... file error {e}', {'e': e})
        sys.exit()


def save_output(localOuput, localWords, localArgs, otherArgs):
    try:
        with open(localOuput, 'w') as outputOpen:
            for line in localWords.myList:
                if isinstance(line[0], str) and line[0][:1] == '#':
                    outputOpen.writelines(str(line) + '\n')
                elif not localArgs.delimiter:
                    outputOpen.writelines(str(line[0]) + '\n')
                else:
                    outputOpen.writelines(str(line[0]) + otherArgs['parse_char'] + str(line[1]) + '\n')
        print_text = 'New list saved to <{color}>{output}</{color}>'
        print_line(print_text, {'output': outputOpen.name})

    except Exception as e:
        print_line('Exiting... file save error {e}', {'e': e})
        sys.exit()


def setup_output(localArgs, otherArgs):
    file_add = otherArgs['file_add'] if 'file_add' in otherArgs else otherArgs['globals']['file_add']

    try:
        # Has an output file been specified? If not, create from either file or URL
        if localArgs.output:
            outputFile = localArgs.output
        elif localArgs.input or localArgs.urllist:
            fileName = localArgs.input[0] if localArgs.input else localArgs.urllist
            filePieces = os.path.splitext(fileName)
            outputFile = f'{filePieces[0]}_{file_add}{filePieces[1]}'
        elif localArgs.webpage:
            urlPieces = urllib.parse.urlparse(localArgs.webpage).hostname.split('.')
            if len(urlPieces) > 1:
                outputFile = f'{urlPieces[-2]}_{urlPieces[-1]}_{file_add}.txt'
            else:
                outputFile = f'{urlPieces[0]}_{file_add}.txt'
            # See if a directory has been specified: don't need to check if valid by now
            if localArgs.directory is not None:
                outputFile = os.path.join(localArgs.directory, outputFile)
        else:
            # Nothing to set up
            return

        # Now check if the output file exists and prompt user if it does
        if pathlib.Path(outputFile).is_file():
            prompt_text = 'Output file named <{color}>{output}</{color}> already exists. Overwrite? (Y/N): '
            text = prompt(HTML(prompt_text.format(color=IMPACT_COLOR, output=outputFile)))
            if text != 'Y' and text != 'y':
                print_line('Exiting... please enter a different file name at the command line')
                sys.exit()
        return outputFile

    except AttributeError:
        print_line('Exiting... problem setting up output file. Probably a problem with the URL you entered')
        sys.exit()

    except Exception as e:
        print_line('Exiting... setup_output error {e}', {'e': e})
        sys.exit()


def setup_input(localArgs, otherArgs):
    returnWords = []
    parseDict = {}

    # Load input(s)
    if localArgs.input:
        if localArgs.delimiter and 'parse_char' not in otherArgs:
            print_line('Exiting... --delimiter option specified but no parse_char found in configuration file')
            sys.exit()

        for inputFile in localArgs.input:
            fileWords = get_file_content(inputFile, localArgs.delimiter, otherArgs['parse_char'])
            if fileWords:
                returnWords.extend(fileWords)

    if 'webpage' in localArgs and localArgs.webpage:
        if localArgs.container:
            parseDict = create_dict(localArgs.container)
        webScrape = WebExtract(parseDict, localArgs.webextract)
        webScrape.pull_data(localArgs.webpage)
        returnWords.extend(webScrape.returnWords)

    if 'urllist' in localArgs and localArgs.urllist:
        urlList = get_file_content(localArgs.urllist, False)
        if urlList:
            urlLength = len(urlList)
            if localArgs.container:
                parseDict = create_dict(localArgs.container)

            for urlCount, oneUrl in enumerate(urlList, start=1):
                print_text = 'Getting <{color}>{oneUrl}</{color}> ({urlCount} of {urlLength})'
                arg_dict = {
                    'oneUrl': oneUrl[0],
                    'urlCount': urlCount,
                    'urlLength': urlLength,
                }
                print_line(print_text, arg_dict, endText='')
                webScrape = WebExtract(parseDict, localArgs.webextract)
                webScrape.pull_data(oneUrl[0])
                returnWords.extend(webScrape.returnWords)
                if urlCount < urlLength:
                    delay = int(otherArgs['urllist_delay'])
                    print_line('  ...done ...sleeping {delay} seconds', {'delay': delay})
                    time.sleep(delay)
                else:
                    print_line('  ...done')

    if len(returnWords) == 0:
        print_text = 'No input given, nothing to do (enter <{color}>{exec_name} -h</{color}> for help)'
        print_line(print_text, {'exec_name': FILE['name']})
        sys.exit()

    return returnWords


def do_word_options(localWords, localArgs, defaultArgs):
    thatOption = 'line2word'
    if localArgs.line2word and localArgs.word2word:
        print_text = 'Options <{color}>line2word</{color}> and <{color}>word2word</{color}>'
        print_text += ' are mutually exclusive, please use either but not both'
        print_line(print_text)
        sys.exit()
    elif localArgs.word2word:
        # If word2word we first do a convert
        localWords.convert(defaultArgs['convert'])
        thatOption = 'word2word'
    # Now run through all the other options
    optionList = ['strip', 'minimum', 'case', 'dedupe', 'alphabetize']
    for option in optionList:
        getattr(localWords, option)(defaultArgs[option])

    return localWords, thatOption


def do_ignore(whichOption, thatOption):
    print_text = 'Option <{color}>{whichOption}</{color}> ignored due to use of <{color}>{thatOption}</{color}>'
    arg_dict = {
        'whichOption': whichOption,
        'thatOption': thatOption,
    }
    print_line(print_text, arg_dict)


def main():
    # First read config file and set up defaults
    defArgs = init_config()
    global IMPACT_COLOR
    IMPACT_COLOR = defArgs['globals']['impact_color']

    # First set up configargparse
    parser = argparse.ArgumentParser(description='Crossword puzzle word list builder')

    # Meta options
    parser.add_argument('-v', '--version', action='version', version=f"{FILE['name']} {__version__}")

    # Input and output options
    parser.add_argument('-i', '--input', nargs='*', type=pathlib.Path, help='Input one or more text file(s)')

    parser.add_argument('-w', '--webpage', help='Input web URL')

    parser.add_argument('--urllist', type=pathlib.Path, help='Input multiple URLs in a document')

    output_help = 'Output text file: if no name specified, a default name is created'
    parser.add_argument('-o', '--output', type=pathlib.Path, help=output_help)

    directory_help = 'Set directory for input, output and urllist files'
    parser.add_argument('--directory', type=pathlib.Path, help=directory_help)

    # Content parsing options
    container_help = 'Further refines the text from a webpage by narrowing to any HTML entity(ies) specified,\
                      using tag=term syntax (e.g., id=main_content or class=lyrics).'
    parser.add_argument('--container', nargs=1, help=container_help)

    webextract_help = 'Specify what to extract from web inputs '
    webextract_help += f"[ text | links | html-XX ] - DEFAULT: {defArgs['defaults']['webextract']}"
    parser.add_argument('--webextract', nargs='?', default=defArgs['defaults']['webextract'], help=webextract_help)

    parser.add_argument('--regex', nargs=1, help='Parse text based on regex')

    delimiter_help = 'Each line contains a delimiter, separate into two columns based on that character '
    delimiter_help += f"- DEFAULT: {defArgs['globals']['parse_char']}"
    parser.add_argument('--delimiter', action='store_true', help=delimiter_help)

    # List transformation options
    line2word_help = 'Convert each line of text into a word by stripping out spaces and other non-alphabetic\
                      characters, dedupe the list, alphabetize and capitalize every word. Remove all words\
                      below the minimum word size. This is the equivalent of running xwordlist with each of those\
                      options set and saves having to enter each one individually. See the help documentation to\
                      better understand how this works.'
    parser.add_argument('--line2word', action='store_true', help=line2word_help)

    word2word_help = 'Convert blocks of text into individual words delimited by spaces, strip out non-alphabetic\
                      characters, dedupe the list, alphabetize and capitalize every word. Remove all words\
                      below the minimum word size. This is the equivalent of running xwordlist with each of those\
                      options set and saves having to enter each one individually. See the help documentation to\
                      better understand how this works.'
    parser.add_argument('--word2word', action='store_true', help=word2word_help)

    convert_help = 'Convert a block of text into individual words, separating words by spaces '
    parser.add_argument('--convert', nargs='?', const=defArgs['defaults']['convert'], help=convert_help)

    alphabetize_help = 'Alphabetize the list '
    alphabetize_help += f"[ normal | reverse ] - DEFAULT: {defArgs['defaults']['alphabetize']}"
    parser.add_argument('-a', '--alphabetize', nargs='?', const=defArgs['defaults']['alphabetize'], help=alphabetize_help)

    case_help = 'Change the case of words in the list '
    case_help += f"[ upper | lower | none ] - DEFAULT: {defArgs['defaults']['case']}"
    parser.add_argument('--case', nargs='?', const=defArgs['defaults']['case'], help=case_help)

    dedupe_help = 'Remove duplicates from the word list '
    dedupe_help += f"[ nocase | bycase ] - DEFAULT: {defArgs['defaults']['dedupe']}"
    parser.add_argument('-d', '--dedupe', nargs='?', const=defArgs['defaults']['dedupe'], help=dedupe_help)

    minimum_help = f"Minimum number of letters in a word - DEFAULT: {defArgs['defaults']['minimum']}"
    parser.add_argument('-m', '--minimum', nargs='?', const=defArgs['defaults']['minimum'], help=minimum_help)

    strip_help = 'Remove non-alphabetic characters '
    strip_help += f"[ diacritic | keepdiacritic ] - DEFAULT: {defArgs['defaults']['strip']}"
    parser.add_argument('-s', '--strip', nargs='?', const=defArgs['defaults']['strip'], help=strip_help)

    confArgs = parser.parse_args()

    # See if a default directory was specified and rewrite inputs and outputs as necessary
    if confArgs.directory is not None or 'directory' in defArgs['globals']:
        directory = confArgs.directory if confArgs.directory else defArgs['globals']['directory']
        if pathlib.Path(directory).is_dir():
            if confArgs.input:
                inputFiles = []
                for inputFile in confArgs.input:
                    inputFiles.append(os.path.join(directory, inputFile) if inputFile else None)
                confArgs.input = list(inputFiles)
            confArgs.output = os.path.join(directory, confArgs.output) if confArgs.output else None
            if 'urllist' in confArgs and confArgs.urllist:
                confArgs.urllist = os.path.join(directory, confArgs.urllist)
        else:
            print_text = 'Exiting... directory path <{color}>{directory}</{color}> does not exist'
            print_line(print_text, {'directory': directory})
            sys.exit()

    # Set up output file to make sure it doesn't exist then grab all inputs (including web parsing)
    outputFile = setup_output(confArgs, defArgs)
    try:

        # Do any text parsing
        if confArgs.regex is not None:
            if confArgs.regex[0] == 'true':
                regex_error = 'Exiting... no regex pattern given, please use format '
                regex_error += '<{color}>regex PATTERN</{color}> in configuration file'
                print_line(regex_error)
                sys.exit()
            inputWords.regex(confArgs.regex[0])

        # First figure out if they set either combination option
        if confArgs.line2word or confArgs.word2word:
            inputWords, thatOption = do_word_options(inputWords, confArgs, defArgs['defaults'])
        else:
            thatOption = False

        # Now run through remaining options
        # We also trap for the case where the user had one of the line2word or word2word options
        # but also uses one of the duplicative other options (we ignore the duplicative one)
        optionList = ['convert', 'strip', 'minimum', 'case', 'dedupe', 'alphabetize']
        for option in optionList:
            confOption = getattr(confArgs, option)
            if confOption is not None:
                do_ignore(option, thatOption) if thatOption else getattr(inputWords, option)(confOption)

        inputWords = WordList(myList=setup_input(confArgs, defArgs['globals']))
    except XWLException as e:
        (err_info, ) = e.args
        if 'arg' in err_info:
            arg = f"<{IMPACT_COLOR}>{err_info['arg']}</{IMPACT_COLOR}>"
            print_formatted_text(HTML(err_info['error'].format(arg)))
        else:
            print_formatted_text(HTML(err_info['error']))
        sys.exit()

    # Now save the file
    save_output(outputFile, inputWords, confArgs, defArgs['globals'])


if __name__ == '__main__':
    main()
